/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.extension.base;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.ui.extension.IContextMenuContributor;
import org.fortiss.tooling.kernel.ui.extension.base.EObjectActionBase.EObjectActionFactory;
import org.fortiss.tooling.kernel.ui.extension.data.ContextMenuContextProvider;
import org.fortiss.tooling.kernel.ui.service.IContextMenuService;

/**
 * Base class for context menu contributors creating a sub-menu
 * {@link IMenuManager} with actions provided by actions factories TODO: @review
 * this link is wrong => {@link ActionFactory} returned by
 * {@link #getActionFactories()}.
 * 
 * The class implements interface methods {@link #getMenuIcon()} and
 * {@link #getMenuId()} returning {@code null} by default. Classes providing an
 * own icon or id should override them.
 * 
 * The generic type parameter {@code T} indicated the type of selection
 * elements.
 * 
 * @param <T>
 *            TODO document T here
 * 
 * @author trachtenherz
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: DE788BDCC0598D12129DAA00E06DDE3E
 */
public abstract class ContextMenuSubMenuContributorBase<T extends EObject>
		implements IContextMenuContributor {
	// TODO reorder methods: implemented before abstract before default
	// implementations
	/** The name of the sub-menu */
	protected abstract String getMenuName();

	/** Returns the action factories for the actions to be shown in the menu */
	protected abstract Collection<EObjectActionFactory<? super T>> getActionFactories();

	/** Creates action instances for the given selection. */
	protected Collection<Action> createActionsForElement(T selection) {
		ArrayList<Action> list = new ArrayList<Action>();
		for (EObjectActionFactory<? super T> factory : getActionFactories()) {
			list.add(factory.createAction(selection));
		}
		return list;
	}

	/** The image for the sub-menu */
	protected ImageDescriptor getMenuIcon() {
		return null;
	}

	/** The Id for the sub-menu */
	protected String getMenuId() {
		return null;
	}

	/**
	 * Creates a sub-menu {@link IMenuManager} containing specified actions.
	 */
	protected IMenuManager createMenu(Collection<Action> actions) {
		IMenuManager menu = new MenuManager(getMenuName(), getMenuIcon(),
				getMenuId());
		for (Action a : actions) {
			menu.add(a);
		}
		return menu;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public final List<IContributionItem> getContributedItems(EObject selection,
			ContextMenuContextProvider contextProvider) {
		if (acceptSelection(selection, contextProvider)) {
			List<IContributionItem> contributionItems = new ArrayList<IContributionItem>();
			contributionItems
					.add(createMenu(createActionsForElement((T) selection)));
			return contributionItems;
		}
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public String getMenuSectionID() {
		return IContextMenuService.BOTTOM_MOST_MENU_SECTION_ID;
	}

	/**
	 * Returns {@code true} if the selected element is acceptable. This method
	 * must also ensure that {@code selection} is an instance of {@code T}
	 */
	protected abstract boolean acceptSelection(EObject selection,
			ContextMenuContextProvider contextProvider);
}